# -*- coding: utf-8 -*-
"""
Created on Sun Dec 27 23:22:53 2020

@author: Johannes Neumeyer
"""

from random import randint
from graphics_and_games_klassen import Figur, Ereignisbehandlung, Text


class Stein (Figur):
    """
    Die Spielfigur, ein Stein
    """

    def __init__(self):
        """
        Legt die Form des Steins fest und positioniert ihn
        """
        super().__init__()
        self.FigurteilFestlegenEllipse(-50, -50, 100, 100, "grau")
        self.PositionSetzen(25, 250)

    def TasteGedrueckt(self, taste):
        """
        Die  Aktionsmethode für gedrückte Sondertasten.

        -- Parameter taste KeyCode der gedrückten Taste
        """
        # Pfeil nach links
        if taste == 'left':
            self.WinkelSetzen(180)
            self.Gehen(10)

        # Pfeil nach oben
        elif taste == 'up':
            self.WinkelSetzen(90)
            self.Gehen(10)

        # Pfeil nach rechts
        elif taste == 'right':
            self.WinkelSetzen(0)
            self.Gehen(10)

        # Pfeil nach unten
        elif taste == 'down':
            self.WinkelSetzen(270)
            self.Gehen(10)

class Steinfresser(Figur):
    """
    Steinfresser
    """

    def __init__(self):
        """
        Legt die Form des Steinfressers fest
        """
        super().__init__()
        self.SpielfigurFestlegen()

    def StartpositionEinnehmen(self):
        """
        Stellt den Steinfresser auf seine Startposition
        """
        self.PositionSetzen(randint(100, 750), randint(50, 500))

    def Bewegen(self):
        """
        Bewegt den Steinfresser
        """
        self.Gehen(25)
        self.Drehen(18)

    def SpielfigurFestlegen(self):
        """
        Beschreibt die Form des Steinfressers
        """
        self.FigurteilFestlegenEllipse(-50, -50, 100, 100, "blau")
        self.FigurteilFestlegenEllipse(-20, 10, 40, 20, "rot")
        self.FigurteilFestlegenEllipse(-20, -20, 5, 5, "gelb")
        self.FigurteilFestlegenEllipse(20, -20, 5, 5, "gelb")


class Steinzeit(Ereignisbehandlung):
    """
    Spielfeld des Steinfresserspiels
    """

    def __init__(self):
        """
        Legt das Spielfeld und die Objekte an
        """
        super().__init__()

        self.spielfigur = Stein()

        # Gegner erstellen
        self.sf1 = Steinfresser()
        self.sf2 = Steinfresser()
        self.sf3 = Steinfresser()
        self.sf4 = Steinfresser()
        self.sf5 = Steinfresser()
        self.sf6 = Steinfresser()
        self.sf7 = Steinfresser()
        self.sf8 = Steinfresser()

        # Alle Gegner sollen ihre (zufällige) Startposition einnehmen
        self.sf1.StartpositionEinnehmen()
        self.sf2.StartpositionEinnehmen()
        self.sf3.StartpositionEinnehmen()
        self.sf4.StartpositionEinnehmen()
        self.sf5.StartpositionEinnehmen()
        self.sf6.StartpositionEinnehmen()
        self.sf7.StartpositionEinnehmen()
        self.sf8.StartpositionEinnehmen()

        self.TaktdauerSetzen(100)
        self.Starten()


    def AktionAusfuehren (self):
        """
        Überschriebene Methode von Ereignisbehandlung,
        die automatisch bei jedem Taktschlag aufgerufen wird
        """
        # Bei jedem Taktschlag sollen sich alle Gegner weiterbewegen
        self.sf1.Bewegen()
        self.sf2.Bewegen()
        self.sf3.Bewegen()
        self.sf4.Bewegen()
        self.sf5.Bewegen()
        self.sf6.Bewegen()
        self.sf7.Bewegen()
        self.sf8.Bewegen()

        # Nach jedem Taktschlag muss geprüft worden, ob man gewonnen oder verloren hat
        self.GewonnenVerlorenPruefen()

    def GewonnenVerlorenPruefen(self):
        """
        Prüft, ob das Spiel gewonnen oder verloren wurde
        und erstellt einen entsprechenden Text
        """
        # Gewonnen prüfen
        if self.spielfigur.XPositionGeben() > 800:
            self.Anhalten()
            gewonnen = Text()
            gewonnen.TextSetzen("Gewonnen!")
            gewonnen.PositionSetzen(300, 250)
            gewonnen.FarbeSetzen("grün")
            gewonnen.TextGroesseSetzen(40)

        # Verloren prüfen
        if self.spielfigur.Beruehrt():
            self.Anhalten()
            verloren = Text()
            verloren.TextSetzen("Verloren!")
            verloren.PositionSetzen(300, 250)
            verloren.FarbeSetzen("rot")
            verloren.TextGroesseSetzen(40)


sz = Steinzeit()
